@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-__version__ = "5.20.0"
+__version__ = "5.21.0"
 
 import logging
 
@@ -34,5 +34,6 @@ def generate(
     ta_version=None,
     outputdir=None,
     python_binary_name="python3",
+    openapi=True,
 ):
-    build.generate(source, config, ta_version, outputdir, python_binary_name)
+    build.generate(source, config, ta_version, outputdir, python_binary_name, openapi)
